use bevy::prelude::{Color, Component};

use crate::{
    structs::{DrawElement, DrawingSegment, MoveCommand},
    turtle_state::TurtleDraw,
};

use super::state::TurtleState;

/// A default turtle drawing lines each line is one Segment.
#[derive(Component)]
pub struct Turtle {
    state: TurtleState,
}

pub struct FilledTurtle {
    state: TurtleState,
    drawing: Vec<DrawingSegment>,
    color: Color,
}

impl Turtle {
    pub fn begin_fill(self, color: Color) -> FilledTurtle {
        FilledTurtle {
            state: self.state,
            drawing: vec![],
            color,
        }
    }
}

impl TurtleDraw for Turtle {
    fn forward(&mut self, length: crate::datatypes::length::Length) -> &mut Self {
        let move_command = MoveCommand::Forward(length);
        self.state
            .add_segment(DrawingSegment::Single(if self.state.drawing() {
                DrawElement::Draw(move_command)
            } else {
                DrawElement::Move(move_command)
            }));

        self
    }

    fn backward(&mut self, length: crate::datatypes::length::Length) -> &mut Self {
        todo!()
    }

    fn circle(
        &mut self,
        radius: crate::datatypes::length::Length,
        angle: crate::datatypes::angle::Angle<super::Precision>,
    ) -> &mut Self {
        todo!()
    }

    fn goto(&mut self, coordinate: crate::datatypes::Coordinate) -> &mut Self {
        todo!()
    }

    fn home(&mut self) -> &mut Self {
        todo!()
    }

    fn dot(&mut self, size: crate::datatypes::length::Length) -> &mut Self {
        todo!()
    }

    fn stamp(&mut self, size: crate::datatypes::length::Length) -> &mut Self {
        todo!()
    }
}
